@@ -1,14 +1,15 @@
-use shumai::{bench_config, BenchContext, MultiThreadBench};
+use serde_json::Value;
+use shumai::{bench_config, BenchContext, ShumaiBench, ShumaiResult};
 
 #[bench_config]
 pub mod test_config {
-    use serde::Serialize;
+    use serde::{Deserialize, Serialize};
     use shumai::ShumaiConfig;
 
-    #[derive(ShumaiConfig, Serialize, Clone, Debug)]
+    #[derive(ShumaiConfig, Serialize, Deserialize, Clone, Debug)]
     pub struct Foo {
         pub name: String,
-        pub threads: Vec<u64>,
+        pub threads: Vec<usize>,
         pub time: usize,
         #[matrix]
         pub a: usize,
@@ -27,12 +28,13 @@ struct TestBench {
     execution_queue: crossbeam::queue::SegQueue<ExecutionSeq>,
 }
 
-impl MultiThreadBench for TestBench {
+impl ShumaiBench for TestBench {
     type Result = usize;
     type Config = Foo;
 
-    fn load(&self) {
+    fn load(&self) -> Option<Value> {
         self.execution_queue.push(ExecutionSeq::Load);
+        None
     }
 
     fn run(&self, context: BenchContext<Foo>) -> Self::Result {
@@ -64,17 +66,15 @@ fn config() {
 }
 
 #[test]
+#[cfg_attr(miri, ignore)]
 fn runner() {
     let config = test_config::Foo::from_config(std::path::Path::new("tests/benchmark.toml"))
         .expect("Failed to parse config!");
-    let repeat = 3;
+    let repeat = 2;
 
     for c in config.iter() {
         let benchmark = TestBench::default();
-        let result = shumai::run(&benchmark, c, repeat);
-        for r in result.iter() {
-            r.write_json().unwrap();
-        }
+        let _result = shumai::run(&benchmark, c, repeat);
 
         let mut gt = Vec::new();
         gt.push(ExecutionSeq::Load);
@@ -99,3 +99,48 @@ fn runner() {
         }
     }
 }
+
+#[test]
+#[cfg_attr(miri, ignore)]
+fn write_json() {
+    let config = test_config::Foo::from_config(std::path::Path::new("tests/benchmark.toml"))
+        .expect("Failed to parse config!");
+    let repeat = 1;
+
+    for c in config.iter() {
+        let benchmark = TestBench::default();
+        let result = shumai::run(&benchmark, c, repeat);
+        let file_path = result.write_json().unwrap();
+
+        let written_data = std::fs::read_to_string(file_path).unwrap();
+        let result: ShumaiResult<Foo, usize> = serde_json::from_str(&written_data).unwrap();
+        assert_eq!(result.config.time, 1);
+        assert_eq!(result.config.threads, vec![1, 2, 3]);
+        assert_eq!(result.bench_results.len(), 3);
+    }
+}
+
+#[test]
+#[cfg(feature = "perf")]
+#[cfg_attr(miri, ignore)]
+fn simple_perf() {
+    let config = test_config::Foo::from_config(std::path::Path::new("tests/benchmark.toml"))
+        .expect("Failed to parse config!");
+    let repeat = 1;
+
+    let c = config.first().unwrap();
+    let benchmark = TestBench::default();
+    let result = shumai::run(&benchmark, c, repeat);
+    let bench_results = result.bench_results;
+    assert_eq!(bench_results.len(), c.threads.len());
+    for rv in bench_results {
+        let perf = rv.perf.expect("perf should be non-empty");
+        // These counters should all be positive
+        assert!(perf.cycles > 0);
+        assert!(perf.inst > 0);
+        assert!(perf.branch_miss > 0);
+        assert!(perf.branches > 0);
+        assert!(perf.cache_reference > 0);
+        assert!(perf.cache_miss > 0);
+    }
+}
